//AD9850 MAX7219 V2 
//http://infotex58.ru/forum/index.php?topic=969.15

#include <Rotary.h>
#include <LedControl.h>

#define pulseHigh(pin) {digitalWrite(pin, HIGH); digitalWrite(pin, LOW); }

//set DDS pins
#define W_CLK A2   // AD9850 load clock pin (FCLK)
#define FQ_UD A3   // freq update pin (FQUD)
#define DATA  A4   // serial data load pin (DATA)
#define RESET A5   // reset pin (RST) 


Rotary r = Rotary(2,3); // sets the pins the rotary encoder uses.  
LedControl lc=LedControl(7,6,5,1);
const int buttonPin = 4;

int buttonState = 0;
int_fast32_t rx = 10000000; // Starting frequency of VFO 10 MHz
int_fast32_t rx_old;        // variable to hold the updated frequency
int_fast32_t rx_ofs;        // command frequency for DDS incl. possible IF offset 
int_fast32_t step1 = 1000;  // starting VFO update increment in HZ.

float rx_disp;           
int  stp = 2;               // starting with 100Hz steps 
int  stp_old;

////////////////////////////////////////////////////////////
void setup() {    
  pinMode(buttonPin, INPUT);
  digitalWrite(buttonPin, HIGH); 
  lc.shutdown(0,false);
  lc.setIntensity(0,3);
  lc.clearDisplay(0);
for (int i =0; i<16; i++)
    {
      lc.setChar(0,i,'d',false);
      lc.setChar(0,i-1,'d',false);
      lc.setChar(0,i-2,'5',false);
      lc.setChar(0,i-3,'-',false);
      lc.setChar(0,i-4,'2',false);
      lc.setChar(0,i-5,'0',false);
      lc.setChar(0,i-6,'1',false);
      lc.setChar(0,i-7,'6',false);
      lc.setChar(0,i-8,' ',false);
      lc.setChar(0,i-9,' ',false);
      lc.setChar(0,i-10,' ',false);
      lc.setChar(0,i-11,' ',false);
      lc.setChar(0,i-12,' ',false);
      lc.setChar(0,i-13,' ',false);
      lc.setChar(0,i-14,' ',false);
      lc.setChar(0,i-15,' ',false);
       delay(300);
    }
    delay (1000);
  PCICR |= (1 << PCIE2);
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);
  sei();

  pinMode(FQ_UD, OUTPUT);
  pinMode(W_CLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(RESET, OUTPUT); 

  pulseHigh(RESET);
  pulseHigh(W_CLK);
  pulseHigh(FQ_UD);  // this pulse enables serial mode on the AD9850 - Datasheet page 12.
}

////////////////////////////////////////////////////////////  
void loop() {
  read_Step();
  read_Step_Select();
  buttonState = digitalRead(buttonPin);

  if (rx != rx_old)  {    
    rx_disp = rx;
    printNumber(rx_disp); 
    rx_old = rx;
    rx_ofs = rx + 0;             // add IF offset frequency if required    
    sendFrequency(rx_ofs);       // frequency for controlling the DDS
  }  
}

////////////////////////////////////////////////////////////
// frequency calc 
void sendFrequency(double frequency) {  
  int32_t freq = frequency * 4294967295/125000000;  // note 125 MHz clock on AD9850.  You can make 'slight' tuning variations here by adjusting the clock frequency.
  for (int b=0; b<4; b++, freq>>=8) {
    tfr_byte(freq & 0xFF);
  }
  tfr_byte(0x000);   // Final control byte, all 0 for AD9850 chip
  pulseHigh(FQ_UD);  // Done!  Should see output
}

////////////////////////////////////////////////////////////
// transfers a byte, a bit at a time, LSB first to the AD9850 via serial DATA line
void tfr_byte(byte data) {
  for (int i=0; i<8; i++, data>>=1) 
  {
    digitalWrite(DATA, data & 0x01);
    pulseHigh(W_CLK);   //after each bit sent, CLK is pulsed high
  }
}

/////////////////////////////////////////////////////////////////
void read_Step_Select() {
  if(stp==8)  
  {
    stp=1; 
  }
  if(stp==0)  
  {
    stp=7; 
  }

  if(stp!=stp_old)  
  {
    if (stp==1) 
    {   
      step1 = 1;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
        
    }
    else if (stp==2) 
    {   
      step1 = 10;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
    }
    else if (stp==3) 
    {   
      step1 = 100;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
      lc.setChar(0,5,'0',false);
    }
    else if (stp==4) 
    {   
      step1 = 1000;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
      lc.setChar(0,5,'0',false);
      lc.setChar(0,4,'0',false);
    }
    else if (stp==5) 
    {   
      step1 = 10000;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
      lc.setChar(0,5,'0',false);
      lc.setChar(0,4,'0',false);
      lc.setChar(0,3,'0',false);
    }
    else if (stp==6) 
    {   
      step1 = 100000;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
      lc.setChar(0,5,'0',false);
      lc.setChar(0,4,'0',false);
      lc.setChar(0,3,'0',false);
      lc.setChar(0,2,'0',false);
    }
     else if (stp==7) 
    {   
      step1 = 1000000;
      lc.clearDisplay(0);
      lc.setChar(0,7,'1',false);
      lc.setChar(0,6,'0',false);
      lc.setChar(0,5,'0',false);
      lc.setChar(0,4,'0',false);
      lc.setChar(0,3,'0',false);
      lc.setChar(0,2,'0',false);
      lc.setChar(0,1,'0',false);
    } 
    delay(500); 
    stp_old = stp;  
  }
}

int read_Step()
{
  if (buttonState == LOW) return stp=stp+1;
}

ISR(PCINT2_vect) {
  unsigned char result = r.process();
  if (result) {    
    if (result == DIR_CW){
      rx=rx+step1;
    }
    else {
      rx=rx-step1;
    };       
    if (rx >=40000000){
      rx=rx_old;
    }; // UPPER VFO LIMIT 40MHz
    if  (rx <= 100000){
      rx=rx_old;
    }; // LOWER VFO LIMIT 100KHz
  }
}

void printNumber(long v) 
{

  int digito1;
  int digito2;
  int digito3;
  int digito4;
  int digito5;
  int digito6;
  int digito7;
  int digito8;


  digito1 = v%10;
  digito2 = (v / 10) % 10;
  digito3 = (v/ 100) % 10;  
  digito4 = (v/1000) %10;
  digito5 = (v/10000) %10;
  digito6 = (v/100000) %10;
  digito7 = (v/1000000) %10;
  digito8 = (v/10000000) %10;


  lc.setDigit(0,7,(byte)digito8,false);
  lc.setDigit(0,6,(byte)digito7,false);
  lc.setDigit(0,5,(byte)digito6,false);
  lc.setDigit(0,4,(byte)digito5,false);
  lc.setDigit(0,3,(byte)digito4,false);
  lc.setDigit(0,2,(byte)digito3,false);
  lc.setDigit(0,1,(byte)digito2,false);
  lc.setDigit(0,0,(byte)digito1,false);
 }
